# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.delete_response import DeleteResponse
from ..types.namespace import Namespace
from ..types.slug_availability_response import SlugAvailabilityResponse
from .raw_client import AsyncRawNamespacesClient, RawNamespacesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class NamespacesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawNamespacesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawNamespacesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawNamespacesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        org_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        page_number: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Namespace]:
        """
        List all namespaces the user has access to, optionally filtered by org.

        Parameters
        ----------
        org_id : typing.Optional[str]
            Filter by owner organization ID

        limit : typing.Optional[int]
            Maximum number of results

        page_number : typing.Optional[int]
            Page number

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Namespace]
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.namespaces.list()
        """
        _response = self._raw_client.list(
            org_id=org_id, limit=limit, page_number=page_number, request_options=request_options
        )
        return _response.data

    def create(
        self, *, name: str, owner_org_id: str, slug: str, request_options: typing.Optional[RequestOptions] = None
    ) -> Namespace:
        """
        Create a new namespace for multi-tenant isolation.

        Parameters
        ----------
        name : str
            Human-readable name.

        owner_org_id : str
            WorkOS organization ID that owns this namespace.

        slug : str
            URL-friendly unique identifier (lowercase alphanumeric and hyphens).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.namespaces.create(
            name="name",
            owner_org_id="owner_org_id",
            slug="slug",
        )
        """
        _response = self._raw_client.create(
            name=name, owner_org_id=owner_org_id, slug=slug, request_options=request_options
        )
        return _response.data

    def check_slug_availability(
        self, slug: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SlugAvailabilityResponse:
        """
        Check if a namespace slug is available for use.

        Parameters
        ----------
        slug : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SlugAvailabilityResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.namespaces.check_slug_availability(
            slug="slug",
        )
        """
        _response = self._raw_client.check_slug_availability(slug, request_options=request_options)
        return _response.data

    def retrieve_by_slug(self, slug: str, *, request_options: typing.Optional[RequestOptions] = None) -> Namespace:
        """
        Get a namespace by its slug.

        Parameters
        ----------
        slug : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.namespaces.retrieve_by_slug(
            slug="slug",
        )
        """
        _response = self._raw_client.retrieve_by_slug(slug, request_options=request_options)
        return _response.data

    def retrieve(self, namespace_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Namespace:
        """
        Get a namespace by ID.

        Parameters
        ----------
        namespace_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.namespaces.retrieve(
            namespace_id="namespace_id",
        )
        """
        _response = self._raw_client.retrieve(namespace_id, request_options=request_options)
        return _response.data

    def delete(self, namespace_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteResponse:
        """
        Delete a namespace (must be empty).

        Parameters
        ----------
        namespace_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.namespaces.delete(
            namespace_id="namespace_id",
        )
        """
        _response = self._raw_client.delete(namespace_id, request_options=request_options)
        return _response.data

    def update(
        self,
        namespace_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Namespace:
        """
        Update a namespace's properties.

        Parameters
        ----------
        namespace_id : str

        name : typing.Optional[str]
            Human-readable name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.namespaces.update(
            namespace_id="namespace_id",
        )
        """
        _response = self._raw_client.update(namespace_id, name=name, request_options=request_options)
        return _response.data

    def retry_provisioning(
        self,
        namespace_id: str,
        *,
        force: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Namespace:
        """
        Retry provisioning for a namespace that failed or is stuck.

        Parameters
        ----------
        namespace_id : str

        force : typing.Optional[bool]
            Force retry even if status is PENDING (for stuck namespaces)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.namespaces.retry_provisioning(
            namespace_id="namespace_id",
        )
        """
        _response = self._raw_client.retry_provisioning(namespace_id, force=force, request_options=request_options)
        return _response.data


class AsyncNamespacesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawNamespacesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawNamespacesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawNamespacesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        org_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        page_number: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Namespace]:
        """
        List all namespaces the user has access to, optionally filtered by org.

        Parameters
        ----------
        org_id : typing.Optional[str]
            Filter by owner organization ID

        limit : typing.Optional[int]
            Maximum number of results

        page_number : typing.Optional[int]
            Page number

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Namespace]
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.namespaces.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            org_id=org_id, limit=limit, page_number=page_number, request_options=request_options
        )
        return _response.data

    async def create(
        self, *, name: str, owner_org_id: str, slug: str, request_options: typing.Optional[RequestOptions] = None
    ) -> Namespace:
        """
        Create a new namespace for multi-tenant isolation.

        Parameters
        ----------
        name : str
            Human-readable name.

        owner_org_id : str
            WorkOS organization ID that owns this namespace.

        slug : str
            URL-friendly unique identifier (lowercase alphanumeric and hyphens).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.namespaces.create(
                name="name",
                owner_org_id="owner_org_id",
                slug="slug",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name, owner_org_id=owner_org_id, slug=slug, request_options=request_options
        )
        return _response.data

    async def check_slug_availability(
        self, slug: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SlugAvailabilityResponse:
        """
        Check if a namespace slug is available for use.

        Parameters
        ----------
        slug : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SlugAvailabilityResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.namespaces.check_slug_availability(
                slug="slug",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.check_slug_availability(slug, request_options=request_options)
        return _response.data

    async def retrieve_by_slug(
        self, slug: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Namespace:
        """
        Get a namespace by its slug.

        Parameters
        ----------
        slug : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.namespaces.retrieve_by_slug(
                slug="slug",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve_by_slug(slug, request_options=request_options)
        return _response.data

    async def retrieve(
        self, namespace_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Namespace:
        """
        Get a namespace by ID.

        Parameters
        ----------
        namespace_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.namespaces.retrieve(
                namespace_id="namespace_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(namespace_id, request_options=request_options)
        return _response.data

    async def delete(
        self, namespace_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteResponse:
        """
        Delete a namespace (must be empty).

        Parameters
        ----------
        namespace_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.namespaces.delete(
                namespace_id="namespace_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(namespace_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        namespace_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Namespace:
        """
        Update a namespace's properties.

        Parameters
        ----------
        namespace_id : str

        name : typing.Optional[str]
            Human-readable name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.namespaces.update(
                namespace_id="namespace_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(namespace_id, name=name, request_options=request_options)
        return _response.data

    async def retry_provisioning(
        self,
        namespace_id: str,
        *,
        force: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Namespace:
        """
        Retry provisioning for a namespace that failed or is stuck.

        Parameters
        ----------
        namespace_id : str

        force : typing.Optional[bool]
            Force retry even if status is PENDING (for stuck namespaces)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Namespace
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.namespaces.retry_provisioning(
                namespace_id="namespace_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retry_provisioning(
            namespace_id, force=force, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class LogStreamErrorEvent(UniversalBaseModel):
    """
    Sent when an error occurs in the log stream.

    After this event, the stream may close or attempt recovery.
    """

    message: str = pydantic.Field()
    """
    Error description
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.build_prepare_response import BuildPrepareResponse
from ..types.build_status_response import BuildStatusResponse
from ..types.build_submit_response import BuildSubmitResponse
from ..types.http_validation_error import HttpValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBuildsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def prepare(
        self, *, agent_name: str, namespace: str, tag: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BuildPrepareResponse]:
        """
        Return a presigned URL for uploading a build context archive to GCS.

        Parameters
        ----------
        agent_name : str
            Agent short name (without namespace prefix)

        namespace : str
            Target namespace slug for the build

        tag : str
            Image tag to build/push (typically git hash)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BuildPrepareResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "builds/prepare",
            method="POST",
            json={
                "agent_name": agent_name,
                "namespace": namespace,
                "tag": tag,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BuildPrepareResponse,
                    parse_obj_as(
                        type_=BuildPrepareResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def submit(
        self,
        *,
        agent_name: str,
        bucket: str,
        namespace: str,
        object_key: str,
        tag: str,
        git_branch: typing.Optional[str] = OMIT,
        git_hash: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BuildSubmitResponse]:
        """
        Submit a Cloud Build build for an uploaded build context archive.

        Parameters
        ----------
        agent_name : str
            Agent short name

        bucket : str
            GCS bucket containing uploaded build context archive.

        namespace : str
            Target namespace slug

        object_key : str
            GCS object key for the uploaded build context archive.

        tag : str
            Image tag to build/push

        git_branch : typing.Optional[str]
            Optional git branch name for metadata.

        git_hash : typing.Optional[str]
            Optional git hash for metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BuildSubmitResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "builds/submit",
            method="POST",
            json={
                "agent_name": agent_name,
                "bucket": bucket,
                "git_branch": git_branch,
                "git_hash": git_hash,
                "namespace": namespace,
                "object_key": object_key,
                "tag": tag,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BuildSubmitResponse,
                    parse_obj_as(
                        type_=BuildSubmitResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def retrieve(
        self, build_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BuildStatusResponse]:
        """
        Get build status for a Cloud Build build ID (proxied).

        Parameters
        ----------
        build_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BuildStatusResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"builds/{jsonable_encoder(build_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BuildStatusResponse,
                    parse_obj_as(
                        type_=BuildStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBuildsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def prepare(
        self, *, agent_name: str, namespace: str, tag: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BuildPrepareResponse]:
        """
        Return a presigned URL for uploading a build context archive to GCS.

        Parameters
        ----------
        agent_name : str
            Agent short name (without namespace prefix)

        namespace : str
            Target namespace slug for the build

        tag : str
            Image tag to build/push (typically git hash)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BuildPrepareResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "builds/prepare",
            method="POST",
            json={
                "agent_name": agent_name,
                "namespace": namespace,
                "tag": tag,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BuildPrepareResponse,
                    parse_obj_as(
                        type_=BuildPrepareResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def submit(
        self,
        *,
        agent_name: str,
        bucket: str,
        namespace: str,
        object_key: str,
        tag: str,
        git_branch: typing.Optional[str] = OMIT,
        git_hash: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BuildSubmitResponse]:
        """
        Submit a Cloud Build build for an uploaded build context archive.

        Parameters
        ----------
        agent_name : str
            Agent short name

        bucket : str
            GCS bucket containing uploaded build context archive.

        namespace : str
            Target namespace slug

        object_key : str
            GCS object key for the uploaded build context archive.

        tag : str
            Image tag to build/push

        git_branch : typing.Optional[str]
            Optional git branch name for metadata.

        git_hash : typing.Optional[str]
            Optional git hash for metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BuildSubmitResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "builds/submit",
            method="POST",
            json={
                "agent_name": agent_name,
                "bucket": bucket,
                "git_branch": git_branch,
                "git_hash": git_hash,
                "namespace": namespace,
                "object_key": object_key,
                "tag": tag,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BuildSubmitResponse,
                    parse_obj_as(
                        type_=BuildSubmitResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def retrieve(
        self, build_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BuildStatusResponse]:
        """
        Get build status for a Cloud Build build ID (proxied).

        Parameters
        ----------
        build_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BuildStatusResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"builds/{jsonable_encoder(build_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BuildStatusResponse,
                    parse_obj_as(
                        type_=BuildStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class RegistryAuthResponse(UniversalBaseModel):
    """
    Response containing short-lived registry credentials for pushing agent images.
    """

    expires_at: dt.datetime = pydantic.Field()
    """
    Token expiration timestamp (typically ~1 hour from issuance)
    """

    registry_url: str = pydantic.Field()
    """
    Docker registry host URL (e.g., us-east4-docker.pkg.dev)
    """

    repository: str = pydantic.Field()
    """
    Repository path within the registry (e.g., project-id/agents)
    """

    token: str = pydantic.Field()
    """
    Short-lived OAuth2 access token for docker login
    """

    username: typing.Optional[str] = pydantic.Field(default=None)
    """
    Username for docker login (always 'oauth2accesstoken' for GCP)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""Tests for tool-specific handlers."""

from terminaluse.lib.utils.output import (
    DefaultOutputHandler,
    OutputHandler,
    RequestHandler,
    TaskOutputHandler,
    TaskRequestHandler,
    default_output_handlers,
    default_request_handlers,
)


class TestTaskRequestHandler:
    """Tests for TaskRequestHandler."""

    def test_implements_protocol(self) -> None:
        """TaskRequestHandler implements RequestHandler protocol."""
        handler = TaskRequestHandler()
        assert isinstance(handler, RequestHandler)

    def test_formats_full_task_request(self) -> None:
        """Formats Task request with all fields."""
        handler = TaskRequestHandler()

        result = handler.format(
            {
                "subagent_type": "Explore",
                "description": "Search for auth patterns",
                "name": "auth-explorer",
                "prompt": "Find all authentication...",
            }
        )

        assert "🤖" in result
        assert "[Explore]" in result
        assert "auth-explorer" in result
        assert "Search for auth patterns" in result
        # Should NOT show the full prompt
        assert "Find all authentication" not in result

    def test_formats_without_name(self) -> None:
        """Formats Task request without optional name."""
        handler = TaskRequestHandler()

        result = handler.format(
            {
                "subagent_type": "Plan",
                "description": "Design the architecture",
            }
        )

        assert "🤖" in result
        assert "[Plan]" in result
        assert "Design the architecture" in result

    def test_formats_without_description(self) -> None:
        """Formats Task request without description."""
        handler = TaskRequestHandler()

        result = handler.format(
            {
                "subagent_type": "general-purpose",
                "name": "helper",
            }
        )

        assert "🤖" in result
        assert "[general-purpose]" in result
        assert "helper" in result

    def test_defaults_subagent_type(self) -> None:
        """Defaults subagent_type to 'agent' when missing."""
        handler = TaskRequestHandler()

        result = handler.format({})

        assert "[agent]" in result

    def test_is_frozen_dataclass(self) -> None:
        """TaskRequestHandler is immutable."""
        handler = TaskRequestHandler()
        # Frozen dataclass should be hashable
        assert hash(handler) is not None


class TestDefaultOutputHandler:
    """Tests for DefaultOutputHandler."""

    def test_implements_protocol(self) -> None:
        """DefaultOutputHandler implements OutputHandler protocol."""
        handler = DefaultOutputHandler()
        assert isinstance(handler, OutputHandler)

    def test_renders_empty_lines(self) -> None:
        """Empty lines show empty marker."""
        handler = DefaultOutputHandler()

        result = handler.render([])

        assert "(empty)" in result
        assert "'-->" in result

    def test_renders_single_line(self) -> None:
        """Single line renders inline."""
        handler = DefaultOutputHandler()

        result = handler.render(["Hello World"])

        assert "'--> Hello World" in result

    def test_renders_multiple_lines(self) -> None:
        """Multiple lines render with arrow prefix."""
        handler = DefaultOutputHandler()

        result = handler.render(["Line 1", "Line 2", "Line 3"])

        assert "'-->" in result
        assert "Line 1" in result
        assert "Line 2" in result
        assert "Line 3" in result

    def test_truncates_when_not_verbose(self) -> None:
        """Truncates lines beyond max_lines."""
        handler = DefaultOutputHandler(max_lines=2, verbose=False)
        lines = [f"line{i}" for i in range(10)]

        result = handler.render(lines)

        assert "line0" in result
        assert "line1" in result
        assert "line9" not in result
        assert "more lines" in result

    def test_shows_all_when_verbose(self) -> None:
        """Verbose mode shows all lines."""
        handler = DefaultOutputHandler(max_lines=2, verbose=True)
        lines = [f"line{i}" for i in range(10)]

        result = handler.render(lines)

        assert "line0" in result
        assert "line9" in result
        assert "more lines" not in result

    def test_is_frozen_dataclass(self) -> None:
        """DefaultOutputHandler is immutable."""
        handler = DefaultOutputHandler()
        assert hash(handler) is not None


class TestTaskOutputHandler:
    """Tests for TaskOutputHandler."""

    def test_implements_protocol(self) -> None:
        """TaskOutputHandler implements OutputHandler protocol."""
        handler = TaskOutputHandler()
        assert isinstance(handler, OutputHandler)

    def test_renders_empty_lines(self) -> None:
        """Empty lines show no response marker."""
        handler = TaskOutputHandler()

        result = handler.render([])

        assert "└─" in result
        assert "no response" in result

    def test_renders_single_line(self) -> None:
        """Single line renders with tree prefix."""
        handler = TaskOutputHandler()

        result = handler.render(["Found 5 files"])

        assert "└─ Found 5 files" in result

    def test_renders_multiple_lines(self) -> None:
        """Multiple lines render with tree prefix."""
        handler = TaskOutputHandler()

        result = handler.render(["Line 1", "Line 2", "Line 3"])

        assert "└─" in result
        assert "Line 1" in result
        assert "Line 2" in result
        assert "Line 3" in result

    def test_truncates_when_not_verbose(self) -> None:
        """Truncates lines beyond max_lines."""
        handler = TaskOutputHandler(max_lines=2, verbose=False)
        lines = [f"line{i}" for i in range(10)]

        result = handler.render(lines)

        assert "line0" in result
        assert "line1" in result
        assert "line9" not in result
        assert "more lines" in result

    def test_shows_all_when_verbose(self) -> None:
        """Verbose mode shows all lines."""
        handler = TaskOutputHandler(max_lines=2, verbose=True)
        lines = [f"line{i}" for i in range(10)]

        result = handler.render(lines)

        assert "line0" in result
        assert "line9" in result
        assert "more lines" not in result

    def test_is_frozen_dataclass(self) -> None:
        """TaskOutputHandler is immutable."""
        handler = TaskOutputHandler()
        assert hash(handler) is not None


class TestDefaultHandlerFactories:
    """Tests for handler factory functions."""

    def test_default_request_handlers_is_empty(self) -> None:
        """default_request_handlers returns empty dict by default."""
        handlers = default_request_handlers()

        assert handlers == {}

    def test_default_output_handlers_is_empty(self) -> None:
        """default_output_handlers returns empty dict by default."""
        handlers = default_output_handlers()

        assert handlers == {}

    def test_can_add_task_handler_to_registry(self) -> None:
        """Handlers can be added to registry for opt-in use."""
        handlers = default_request_handlers()
        handlers["Task"] = TaskRequestHandler()

        assert "Task" in handlers
        assert isinstance(handlers["Task"], TaskRequestHandler)

    def test_can_add_output_handler_with_settings(self) -> None:
        """Output handlers can be configured when added."""
        handlers = default_output_handlers()
        handlers["Task"] = TaskOutputHandler(max_lines=5, verbose=True)

        task_handler = handlers["Task"]
        assert task_handler.max_lines == 5
        assert task_handler.verbose is True



"""Git detection utilities for deployment flow.

Detects git information (branch, commit hash, dirty state) from the current directory
to support image tagging and deployment tracking.
"""

from __future__ import annotations

import subprocess
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GitInfo:
    """Git repository information."""

    branch: str | None
    commit_hash: str | None
    commit_message: str | None
    is_dirty: bool
    is_git_repo: bool


def detect_git_info(repo_path: str = ".") -> GitInfo:
    """Detect git information from the specified directory.

    Args:
        repo_path: Path to the repository (default: current directory)

    Returns:
        GitInfo containing branch, commit hash, message, and dirty state
    """
    # Check if this is a git repository
    try:
        subprocess.run(
            ["git", "rev-parse", "--git-dir"],
            cwd=repo_path,
            capture_output=True,
            check=True,
        )
    except (subprocess.CalledProcessError, FileNotFoundError):
        return GitInfo(
            branch=None,
            commit_hash=None,
            commit_message=None,
            is_dirty=False,
            is_git_repo=False,
        )

    # Get current branch
    branch_result = subprocess.run(
        ["git", "rev-parse", "--abbrev-ref", "HEAD"],
        cwd=repo_path,
        capture_output=True,
        text=True,
    )
    branch = branch_result.stdout.strip() if branch_result.returncode == 0 else None

    # Handle detached HEAD state
    if branch == "HEAD":
        branch = None

    # Get full commit hash
    hash_result = subprocess.run(
        ["git", "rev-parse", "HEAD"],
        cwd=repo_path,
        capture_output=True,
        text=True,
    )
    commit_hash = hash_result.stdout.strip() if hash_result.returncode == 0 else None

    # Get commit message (first line only)
    msg_result = subprocess.run(
        ["git", "log", "-1", "--format=%s"],
        cwd=repo_path,
        capture_output=True,
        text=True,
    )
    commit_message = msg_result.stdout.strip() if msg_result.returncode == 0 else None

    # Check for uncommitted changes (dirty state)
    dirty_result = subprocess.run(
        ["git", "status", "--porcelain"],
        cwd=repo_path,
        capture_output=True,
        text=True,
    )
    is_dirty = bool(dirty_result.stdout.strip())

    return GitInfo(
        branch=branch,
        commit_hash=commit_hash,
        commit_message=commit_message,
        is_dirty=is_dirty,
        is_git_repo=True,
    )


def generate_image_tag(git_info: GitInfo) -> str:
    """Generate an image tag from git info or timestamp.

    Args:
        git_info: Git repository information

    Returns:
        Image tag string (commit hash prefix or nogit-timestamp)
    """
    if git_info.is_git_repo and git_info.commit_hash:
        # Use first 12 characters of commit hash
        return git_info.commit_hash[:12]
    else:
        # No git repo - use timestamp-based tag
        return f"nogit-{datetime.now().strftime('%Y%m%d%H%M%S')}"


def get_short_hash(git_info: GitInfo, length: int = 8) -> str | None:
    """Get a short version of the commit hash.

    Args:
        git_info: Git repository information
        length: Number of characters to include (default: 8)

    Returns:
        Short commit hash or None if not available
    """
    if git_info.commit_hash:
        return git_info.commit_hash[:length]
    return None


@dataclass
class GitAuthor:
    """Git author information."""

    name: str | None
    email: str | None


def detect_git_author(repo_path: str = ".") -> GitAuthor:
    """Detect git author information from git config.

    Args:
        repo_path: Path to the repository (default: current directory)

    Returns:
        GitAuthor with name and email from git config
    """
    name = None
    email = None

    # Get author name from git config
    try:
        name_result = subprocess.run(
            ["git", "config", "user.name"],
            cwd=repo_path,
            capture_output=True,
            text=True,
        )
        if name_result.returncode == 0:
            name = name_result.stdout.strip() or None
    except (subprocess.CalledProcessError, FileNotFoundError):
        pass

    # Get author email from git config
    try:
        email_result = subprocess.run(
            ["git", "config", "user.email"],
            cwd=repo_path,
            capture_output=True,
            text=True,
        )
        if email_result.returncode == 0:
            email = email_result.stdout.strip() or None
    except (subprocess.CalledProcessError, FileNotFoundError):
        pass

    return GitAuthor(name=name, email=email)



from __future__ import annotations

from typing import Any

import yaml  # type: ignore[import-untyped]
from yaml.scanner import ScannerError  # type: ignore[import-untyped]


class InvalidYAMLError(ValueError):
    """
    Raised when trying to red a YAML file, but the file is not formatted correctly.
    """


def load_yaml_file(file_path: str) -> dict[str, Any]:
    """
    Loads a YAML file from the specified path.

    :param file_path: The path of the YAML file to load.
    :type file_path: str
    :return: The contents of the YAML file.
    :rtype: dict
    """
    try:
        with open(file_path) as file:
            yaml_dict = yaml.safe_load(file)
        return yaml_dict
    except ScannerError as error:
        raise InvalidYAMLError(f"The following file is not in valid YAML format: {file_path}") from error



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ProjectListItem(UniversalBaseModel):
    """
    Minimal project shape for discovery/listing.

    The /projects list endpoint is authorized via `discover` and must not leak
    read-level fields (e.g. description).
    """

    id: str = pydantic.Field()
    """
    The unique identifier of the project.
    """

    name: str = pydantic.Field()
    """
    Project name.
    """

    namespace_id: str = pydantic.Field()
    """
    The namespace this project belongs to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class AgentMetricsResponse(UniversalBaseModel):
    """
    Aggregated metrics for an agent.
    """

    avg_task_duration_seconds: typing.Optional[float] = pydantic.Field(default=None)
    """
    Average duration of completed/failed tasks in seconds. None if no tasks have finished in the time window.
    """

    error_log_count24h: typing_extensions.Annotated[int, FieldMetadata(alias="error_log_count_24h")] = pydantic.Field(
        alias="error_log_count_24h"
    )
    """
    Number of ERROR/CRITICAL level logs in the last 24 hours
    """

    task_count24h: typing_extensions.Annotated[int, FieldMetadata(alias="task_count_24h")] = pydantic.Field(
        alias="task_count_24h"
    )
    """
    Total number of tasks created in the last 24 hours
    """

    task_success_rate_pct: typing.Optional[float] = pydantic.Field(default=None)
    """
    Percentage of successful tasks (COMPLETED) vs failed tasks. None if no tasks have finished in the time window.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from __future__ import annotations

from enum import Enum

from temporalio import activity

from terminaluse.lib.core.services.adk.events import EventsService
from terminaluse.lib.types.tracing import BaseModelWithTraceParams
from terminaluse.lib.utils.logging import make_logger
from terminaluse.types.event import Event

logger = make_logger(__name__)


class EventsActivityName(str, Enum):
    GET_EVENT = "get-event"
    LIST_EVENTS = "list-events"


class GetEventParams(BaseModelWithTraceParams):
    event_id: str


class ListEventsParams(BaseModelWithTraceParams):
    task_id: str
    agent_id: str
    last_processed_event_id: str | None = None
    limit: int | None = None


class EventsActivities:
    def __init__(self, events_service: EventsService):
        self._events_service = events_service

    @activity.defn(name=EventsActivityName.GET_EVENT)
    async def get_event(self, params: GetEventParams) -> Event | None:
        return await self._events_service.get_event(
            event_id=params.event_id,
            trace_id=params.trace_id,
            parent_span_id=params.parent_span_id,
        )

    @activity.defn(name=EventsActivityName.LIST_EVENTS)
    async def list(self, params: ListEventsParams) -> list[Event]:
        return await self._events_service.list(
            task_id=params.task_id,
            agent_id=params.agent_id,
            last_processed_event_id=params.last_processed_event_id,
            limit=params.limit,
            trace_id=params.trace_id,
            parent_span_id=params.parent_span_id,
        )



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .schedule_action_info import ScheduleActionInfo
from .schedule_spec_info import ScheduleSpecInfo
from .schedule_state import ScheduleState


class ScheduleResponse(UniversalBaseModel):
    """
    Response model for schedule operations
    """

    action: ScheduleActionInfo
    agent_id: str = pydantic.Field()
    """
    ID of the agent this schedule belongs to
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When the schedule was created
    """

    last_action_time: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When the schedule last executed
    """

    name: str = pydantic.Field()
    """
    Human-readable name for the schedule
    """

    next_action_times: typing.Optional[typing.List[dt.datetime]] = pydantic.Field(default=None)
    """
    Upcoming scheduled execution times
    """

    num_actions_missed: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of scheduled executions that were missed
    """

    num_actions_taken: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of times the schedule has executed
    """

    schedule_id: str = pydantic.Field()
    """
    Unique identifier for the schedule
    """

    spec: ScheduleSpecInfo = pydantic.Field()
    """
    Schedule specification
    """

    state: ScheduleState = pydantic.Field()
    """
    Current state of the schedule
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

"""
jsonable_encoder converts a Python object to a JSON-friendly dict
(e.g. datetimes to strings, Pydantic models to dicts).

Taken from FastAPI, and made a bit simpler
https://github.com/tiangolo/fastapi/blob/master/fastapi/encoders.py
"""

import base64
import dataclasses
import datetime as dt
from enum import Enum
from pathlib import PurePath
from types import GeneratorType
from typing import Any, Callable, Dict, List, Optional, Set, Union

import pydantic
from .datetime_utils import serialize_datetime
from .pydantic_utilities import (
    IS_PYDANTIC_V2,
    encode_by_type,
    to_jsonable_with_fallback,
)

SetIntStr = Set[Union[int, str]]
DictIntStrAny = Dict[Union[int, str], Any]


def jsonable_encoder(obj: Any, custom_encoder: Optional[Dict[Any, Callable[[Any], Any]]] = None) -> Any:
    custom_encoder = custom_encoder or {}
    # Generated SDKs use Ellipsis (`...`) as the sentinel value for "OMIT".
    # OMIT values should be excluded from serialized payloads.
    if obj is Ellipsis:
        return None
    if custom_encoder:
        if type(obj) in custom_encoder:
            return custom_encoder[type(obj)](obj)
        else:
            for encoder_type, encoder_instance in custom_encoder.items():
                if isinstance(obj, encoder_type):
                    return encoder_instance(obj)
    if isinstance(obj, pydantic.BaseModel):
        if IS_PYDANTIC_V2:
            encoder = getattr(obj.model_config, "json_encoders", {})  # type: ignore # Pydantic v2
        else:
            encoder = getattr(obj.__config__, "json_encoders", {})  # type: ignore # Pydantic v1
        if custom_encoder:
            encoder.update(custom_encoder)
        obj_dict = obj.dict(by_alias=True)
        if "__root__" in obj_dict:
            obj_dict = obj_dict["__root__"]
        if "root" in obj_dict:
            obj_dict = obj_dict["root"]
        return jsonable_encoder(obj_dict, custom_encoder=encoder)
    if dataclasses.is_dataclass(obj):
        obj_dict = dataclasses.asdict(obj)  # type: ignore
        return jsonable_encoder(obj_dict, custom_encoder=custom_encoder)
    if isinstance(obj, bytes):
        return base64.b64encode(obj).decode("utf-8")
    if isinstance(obj, Enum):
        return obj.value
    if isinstance(obj, PurePath):
        return str(obj)
    if isinstance(obj, (str, int, float, type(None))):
        return obj
    if isinstance(obj, dt.datetime):
        return serialize_datetime(obj)
    if isinstance(obj, dt.date):
        return str(obj)
    if isinstance(obj, dict):
        encoded_dict = {}
        allowed_keys = set(obj.keys())
        for key, value in obj.items():
            if key in allowed_keys:
                if value is Ellipsis:
                    continue
                encoded_key = jsonable_encoder(key, custom_encoder=custom_encoder)
                encoded_value = jsonable_encoder(value, custom_encoder=custom_encoder)
                encoded_dict[encoded_key] = encoded_value
        return encoded_dict
    if isinstance(obj, (list, set, frozenset, GeneratorType, tuple)):
        encoded_list = []
        for item in obj:
            if item is Ellipsis:
                continue
            encoded_list.append(jsonable_encoder(item, custom_encoder=custom_encoder))
        return encoded_list

    def fallback_serializer(o: Any) -> Any:
        attempt_encode = encode_by_type(o)
        if attempt_encode is not None:
            return attempt_encode

        try:
            data = dict(o)
        except Exception as e:
            errors: List[Exception] = []
            errors.append(e)
            try:
                data = vars(o)
            except Exception as e:
                errors.append(e)
                raise ValueError(errors) from e
        return jsonable_encoder(data, custom_encoder=custom_encoder)

    return to_jsonable_with_fallback(obj, fallback_serializer)



from __future__ import annotations

from terminaluse.lib.client import AsyncTerminalUse
from terminaluse.lib.core.tracing.tracer import AsyncTracer
from terminaluse.lib.utils.logging import make_logger
from terminaluse.types.agent_task_tracker import AgentTaskTracker

logger = make_logger(__name__)


class AgentTaskTrackerService:
    def __init__(
        self,
        terminaluse_client: AsyncTerminalUse,
        tracer: AsyncTracer,
    ):
        self._terminaluse_client = terminaluse_client
        self._tracer = tracer

    async def get_agent_task_tracker(
        self,
        tracker_id: str,
        trace_id: str | None = None,
        parent_span_id: str | None = None,
    ) -> AgentTaskTracker:
        trace = self._tracer.trace(trace_id)
        async with trace.span(
            parent_id=parent_span_id,
            name="get_agent_task_tracker",
            input={"tracker_id": tracker_id},
        ) as span:
            tracker = await self._terminaluse_client.tracker.retrieve(tracker_id)
            if span:
                span.output = tracker.model_dump()
            return tracker

    async def get_by_task_and_agent(
        self,
        task_id: str,
        agent_id: str,
        trace_id: str | None = None,
        parent_span_id: str | None = None,
    ) -> AgentTaskTracker | None:
        trace = self._tracer.trace(trace_id)
        async with trace.span(
            parent_id=parent_span_id,
            name="get_by_task_and_agent",
            input={"task_id": task_id, "agent_id": agent_id},
        ) as span:
            trackers = await self._terminaluse_client.tracker.list(
                task_id=task_id,
                agent_id=agent_id,
            )
            tracker = trackers[0] if trackers else None
            if span:
                span.output = tracker.model_dump() if tracker else None
            return tracker

    async def update_agent_task_tracker(
        self,
        tracker_id: str,
        last_processed_event_id: str | None = None,
        status: str | None = None,
        status_reason: str | None = None,
        trace_id: str | None = None,
        parent_span_id: str | None = None,
    ) -> AgentTaskTracker:
        trace = self._tracer.trace(trace_id)
        async with trace.span(
            parent_id=parent_span_id,
            name="update_agent_task_tracker",
            input={
                "tracker_id": tracker_id,
                "last_processed_event_id": last_processed_event_id,
                "status": status,
                "status_reason": status_reason,
            },
        ) as span:
            tracker = await self._terminaluse_client.tracker.update(
                tracker_id=tracker_id,
                last_processed_event_id=last_processed_event_id,
                status=status,
                status_reason=status_reason,
            )
            if span:
                span.output = tracker.model_dump()
            return tracker



"""Claude SDK hooks for tracing subagent execution.

This module provides hook callbacks that integrate with Claude SDK's hooks system
for tracing nested subagent execution in TerminalUse.

NOTE: Tool streaming is now handled through adk.messages.send() which accepts
Claude SDK message types directly. The hooks here are ONLY for tracing purposes.

Usage:
    # Users should send messages directly via adk.messages:
    async for message in query(prompt="...", options=options):
        await adk.messages.send(task_id=task_id, content=message)

    # Hooks are optional and only needed for tracing subagent spans:
    hooks = create_tracing_hooks(task_id, trace_id, parent_span_id)
    options = ClaudeAgentOptions(hooks=hooks)
"""

from __future__ import annotations

from typing import Any

from claude_agent_sdk import HookMatcher

from terminaluse.lib import adk
from terminaluse.lib.utils.logging import make_logger

logger = make_logger(__name__)


class TracingHooks:
    """Hooks for tracing subagent execution in TerminalUse.

    Creates nested trace spans when Claude executes subagent tools (Task tool).
    This allows tracking of subagent execution in the tracing system.

    NOTE: Tool requests/responses are now captured in ClaudeMessageContent
    via adk.messages.send(). These hooks are ONLY for tracing, not streaming.
    """

    def __init__(
        self,
        task_id: str | None,
        trace_id: str | None = None,
        parent_span_id: str | None = None,
    ):
        """Initialize tracing hooks.

        Args:
            task_id: TerminalUse task ID (for logging context)
            trace_id: Trace ID for nested spans
            parent_span_id: Parent span ID for subagent spans
        """
        self.task_id = task_id
        self.trace_id = trace_id
        self.parent_span_id = parent_span_id

        # Track active subagent spans
        self.subagent_spans: dict[str, Any] = {}  # tool_call_id -> (ctx, span)

    async def pre_tool_use(
        self,
        input_data: dict[str, Any],
        tool_use_id: str | None,
        _context: Any,
    ) -> dict[str, Any]:
        """Hook called before tool execution.

        Creates a nested trace span for subagent tools (Task tool).

        Args:
            input_data: Contains tool_name, tool_input from Claude SDK
            tool_use_id: Unique ID for this tool call
            _context: Hook context from Claude SDK

        Returns:
            Empty dict (allow execution to proceed)
        """
        if not tool_use_id:
            return {}

        tool_name = input_data.get("tool_name", "unknown")
        tool_input = input_data.get("tool_input", {})

        logger.debug(f"Tool started: {tool_name}")

        # Create nested trace span for subagent (Task) tools
        if tool_name == "Task" and self.trace_id and self.parent_span_id:
            subagent_type = tool_input.get("subagent_type", "unknown")
            logger.info(f"Subagent started: {subagent_type}")

            subagent_ctx = adk.tracing.span(
                trace_id=self.trace_id,
                parent_id=self.parent_span_id,
                name=f"Subagent: {subagent_type}",
                input=tool_input,
            )
            subagent_span = await subagent_ctx.__aenter__()
            self.subagent_spans[tool_use_id] = (subagent_ctx, subagent_span)

        return {}

    async def post_tool_use(
        self,
        input_data: dict[str, Any],
        tool_use_id: str | None,
        _context: Any,
    ) -> dict[str, Any]:
        """Hook called after tool execution.

        Closes the nested trace span for subagent tools.

        Args:
            input_data: Contains tool_name, tool_output from Claude SDK
            tool_use_id: Unique ID for this tool call
            _context: Hook context from Claude SDK

        Returns:
            Empty dict
        """
        if not tool_use_id:
            return {}

        tool_name = input_data.get("tool_name", "unknown")
        tool_output = input_data.get("tool_output", "")

        logger.debug(f"Tool completed: {tool_name}")

        # Close subagent span if this was a Task tool
        if tool_use_id in self.subagent_spans:
            subagent_ctx, subagent_span = self.subagent_spans[tool_use_id]
            subagent_span.output = {"result": tool_output}
            await subagent_ctx.__aexit__(None, None, None)
            logger.info(f"Subagent completed: {tool_name}")
            del self.subagent_spans[tool_use_id]

        return {}


def create_tracing_hooks(
    task_id: str | None = None,
    trace_id: str | None = None,
    parent_span_id: str | None = None,
) -> dict[str, list[HookMatcher]]:
    """Create Claude SDK hooks for tracing subagent execution.

    These hooks are OPTIONAL and only needed if you want to trace subagent
    execution. Tool requests/responses are captured via adk.messages.send().

    Args:
        task_id: TerminalUse task ID (for logging context)
        trace_id: Trace ID for nested spans
        parent_span_id: Parent span ID for subagent spans

    Returns:
        Dict with PreToolUse and PostToolUse hook configurations

    Example:
        hooks = create_tracing_hooks(task_id, trace_id, parent_span_id)
        options = ClaudeAgentOptions(hooks=hooks)

        async for message in query(prompt="...", options=options):
            await adk.messages.send(task_id=task_id, content=message)
    """
    hooks_instance = TracingHooks(task_id, trace_id, parent_span_id)

    return {
        "PreToolUse": [
            HookMatcher(
                matcher=None,  # Match all tools
                hooks=[hooks_instance.pre_tool_use],  # type: ignore[list-item]
            )
        ],
        "PostToolUse": [
            HookMatcher(
                matcher=None,  # Match all tools
                hooks=[hooks_instance.post_tool_use],  # type: ignore[list-item]
            )
        ],
    }


# Backwards compatibility alias
create_streaming_hooks = create_tracing_hooks
TemporalStreamingHooks = TracingHooks



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SystemFolderTypeResponse(UniversalBaseModel):
    """
    Response model for a system folder type from the registry.
    """

    host_path: str = pydantic.Field()
    """
    Relative path from $DATA_ROOT/tasks/{task_id}/ on the host.
    """

    sandbox_mount_path: str = pydantic.Field()
    """
    Absolute path where this folder is mounted in the sandbox.
    """

    type: str = pydantic.Field()
    """
    The system folder type identifier (e.g., 'dot_claude').
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from pydantic import BaseModel, Field


class CredentialMapping(BaseModel):
    """Maps a Kubernetes secret to an environment variable in the agent container.

    This allows agents to securely access credentials stored in Kubernetes secrets
    by mapping them to environment variables. For example, you can map a secret
    containing an API key to an environment variable that your agent code expects.

    Example:
        A mapping of {"env_var_name": "OPENAI_API_KEY",
                     "secret_name": "ai-credentials",
                     "secret_key": "openai-key"}
        will make the value from the "openai-key" field in the "ai-credentials"
        Kubernetes secret available to the agent as OPENAI_API_KEY environment variable.

    Attributes:
        env_var_name: The name of the environment variable that will be available to the agent
        secret_name: The name of the Kubernetes secret containing the credential
        secret_key: The key within the Kubernetes secret that contains the credential value
    """

    env_var_name: str = Field(
        ...,
        description="Name of the environment variable that will be available to the agent",
    )
    secret_name: str = Field(..., description="Name of the Kubernetes secret containing the credential")
    secret_key: str = Field(
        ...,
        description="Key within the Kubernetes secret that contains the credential value",
    )



from __future__ import annotations

from enum import Enum
from typing import Optional

from temporalio import activity

from terminaluse.lib.core.services.adk.agents import AgentsService
from terminaluse.lib.types.tracing import BaseModelWithTraceParams
from terminaluse.lib.utils.logging import make_logger
from terminaluse.types.agent import Agent

logger = make_logger(__name__)


class AgentsActivityName(str, Enum):
    GET_AGENT = "get-agent"


class GetAgentParams(BaseModelWithTraceParams):
    agent_id: Optional[str] = None
    agent_name: Optional[str] = None


class AgentsActivities:
    def __init__(self, agents_service: AgentsService):
        self._agents_service = agents_service

    @activity.defn(name=AgentsActivityName.GET_AGENT)
    async def get_agent(self, params: GetAgentParams) -> Agent | None:
        return await self._agents_service.get_agent(
            agent_id=params.agent_id,
            agent_name=params.agent_name,
            trace_id=params.trace_id,
            parent_span_id=params.parent_span_id,
        )



from __future__ import annotations

from typing import Any

from terminaluse.lib.core.clients.temporal.temporal_client import TemporalClient
from terminaluse.lib.core.clients.temporal.types import WorkflowState
from terminaluse.lib.core.temporal.types.workflow import SignalName
from terminaluse.lib.environment_variables import EnvironmentVariables
from terminaluse.lib.types.acp import CreateTaskParams, SendEventParams
from terminaluse.types.agent import Agent
from terminaluse.types.event import Event
from terminaluse.types.task import Task


class TemporalTaskService:
    """
    Submits Agent agent_tasks to the async runtime for execution.
    """

    def __init__(
        self,
        temporal_client: TemporalClient,
        env_vars: EnvironmentVariables,
    ):
        self._temporal_client = temporal_client
        self._env_vars = env_vars

    async def submit_task(self, agent: Agent, task: Task, params: dict[str, Any] | None) -> str:
        """
        Submit a task to the async runtime for execution.

        returns the workflow ID of the temporal workflow
        """
        return await self._temporal_client.start_workflow(
            workflow=self._env_vars.TERMINALUSE_WORKFLOW_NAME,
            arg=CreateTaskParams(
                agent=agent,
                task=task,
                params=params,
            ),
            id=task.id,
            task_queue=self._env_vars.TERMINALUSE_WORKFLOW_TASK_QUEUE,
        )

    async def get_state(self, task_id: str) -> WorkflowState:
        """
        Get the task state from the async runtime.
        """
        return await self._temporal_client.get_workflow_status(
            workflow_id=task_id,
        )

    async def send_event(self, agent: Agent, task: Task, event: Event, request: dict | None = None) -> None:
        return await self._temporal_client.send_signal(
            workflow_id=task.id,
            signal=SignalName.RECEIVE_EVENT.value,
            payload=SendEventParams(
                agent=agent,
                task=task,
                event=event,
                request=request,
            ).model_dump(),
        )

    async def cancel(self, task_id: str) -> None:
        return await self._temporal_client.cancel_workflow(
            workflow_id=task_id,
        )

    async def terminate(self, task_id: str) -> None:
        return await self._temporal_client.terminate_workflow(
            workflow_id=task_id,
        )



from __future__ import annotations

from datetime import datetime
from typing import Any

from jinja2 import BaseLoader, Environment

from terminaluse.lib.core.tracing.tracer import AsyncTracer
from terminaluse.lib.utils.temporal import heartbeat_if_in_workflow

# Create a Jinja environment
JINJA_ENV = Environment(
    loader=BaseLoader(),
    trim_blocks=True,
    lstrip_blocks=True,
    extensions=["jinja2.ext.do"],
)


class TemplatingService:
    def __init__(self, tracer: AsyncTracer | None = None):
        self.tracer = tracer

    async def render_jinja(
        self,
        template: str,
        variables: dict[str, Any],
        trace_id: str | None = None,
        parent_span_id: str | None = None,
    ) -> str:
        """
        Activity that renders a Jinja template with the provided data.

        Args:
            template: The template string to render.
            variables: The variables to render the template with.
            trace_id: The trace ID for tracing.
            parent_span_id: The parent span ID for tracing.

        Returns:
            The rendered template as a string
        """
        if self.tracer is None:
            raise RuntimeError("Tracer not initialized - ensure tracer is provided to TemplatingService")
        trace = self.tracer.trace(trace_id)
        async with trace.span(
            parent_id=parent_span_id,
            name="render_jinja",
            input={"template": template, "variables": variables},
        ) as span:
            heartbeat_if_in_workflow("render jinja")
            global_variables = {
                "datetime": datetime,
            }
            jinja_template = JINJA_ENV.from_string(template, globals=global_variables)
            try:
                rendered_template = jinja_template.render(variables)
                if span:
                    span.output = {"jinja_output": rendered_template}
                return rendered_template
            except Exception as e:
                raise ValueError(f"Error rendering Jinja template: {str(e)}") from e



"""CLI telemetry module for OpenTelemetry instrumentation.

This module provides end-to-end trace propagation from CLI to Nucleus to Agent.
Traces are exported to OTLP endpoint if configured, otherwise disabled.

Environment variables:
- TU_TELEMETRY_DISABLED: Set to "1" or "true" to disable CLI telemetry
- OTEL_EXPORTER_OTLP_ENDPOINT: OTLP collector endpoint (e.g., "http://localhost:4317")
- OTEL_EXPORTER_OTLP_TRACES_ENDPOINT: OTLP traces endpoint (takes precedence)

Usage:
    from terminaluse.lib.cli.telemetry import init_cli_telemetry, cli_command_span

    # Initialize once at CLI startup
    init_cli_telemetry()

    # Wrap commands with spans
    with cli_command_span("tasks.create") as span:
        span.set_attribute("task.name", task_name)
        # ... command logic
"""

from __future__ import annotations

import os
from contextlib import contextmanager
from typing import TYPE_CHECKING, Iterator, Optional

if TYPE_CHECKING:
    from opentelemetry.trace import Span, Tracer

# Module-level state
_tracer: Optional["Tracer"] = None
_initialized = False


def _is_telemetry_enabled() -> bool:
    """Check if CLI telemetry is enabled."""
    disabled = os.environ.get("TU_TELEMETRY_DISABLED", "").lower()
    return disabled not in ("1", "true", "yes")


def _get_otlp_endpoint() -> Optional[str]:
    """Get the OTLP endpoint from environment."""
    # Traces-specific endpoint takes precedence
    endpoint = os.environ.get("OTEL_EXPORTER_OTLP_TRACES_ENDPOINT")
    if endpoint:
        return endpoint
    # Fall back to generic endpoint
    return os.environ.get("OTEL_EXPORTER_OTLP_ENDPOINT")


def init_cli_telemetry(
    service_name: str = "tu-cli",
    service_version: Optional[str] = None,
) -> bool:
    """Initialize OpenTelemetry for CLI.

    This sets up a TracerProvider with OTLP export if an endpoint is configured.
    If no OTLP endpoint is set, telemetry is silently disabled (no-op).

    Args:
        service_name: The service name for traces (default: "tu-cli")
        service_version: The service version (default: reads from terminaluse.version)

    Returns:
        True if telemetry was initialized, False if disabled or unavailable
    """
    global _tracer, _initialized

    if _initialized:
        return _tracer is not None

    if not _is_telemetry_enabled():
        _initialized = True
        return False

    otlp_endpoint = _get_otlp_endpoint()
    if not otlp_endpoint:
        # No endpoint configured - silently disable telemetry
        _initialized = True
        return False

    try:
        from opentelemetry import trace
        from opentelemetry.sdk.resources import SERVICE_NAME, SERVICE_VERSION, Resource
        from opentelemetry.sdk.trace import TracerProvider
        from opentelemetry.sdk.trace.export import BatchSpanProcessor

        # Get version if not provided
        if service_version is None:
            try:
                from terminaluse.version import __version__

                service_version = __version__
            except ImportError:
                service_version = "unknown"

        # Create resource with service info
        resource = Resource.create(
            {
                SERVICE_NAME: service_name,
                SERVICE_VERSION: service_version,
                "telemetry.sdk.name": "terminaluse-cli",
                "telemetry.sdk.language": "python",
            }
        )

        # Create tracer provider
        provider = TracerProvider(resource=resource)

        # Configure OTLP exporter
        try:
            from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter

            # Check if endpoint uses gRPC (default) or HTTP
            if "4318" in otlp_endpoint or "/v1/traces" in otlp_endpoint:
                # HTTP endpoint
                from opentelemetry.exporter.otlp.proto.http.trace_exporter import (
                    OTLPSpanExporter as HTTPOTLPSpanExporter,
                )

                exporter = HTTPOTLPSpanExporter(endpoint=otlp_endpoint)
            else:
                # gRPC endpoint (default)
                exporter = OTLPSpanExporter(
                    endpoint=otlp_endpoint,
                    insecure=not otlp_endpoint.startswith("https://"),
                )

            provider.add_span_processor(BatchSpanProcessor(exporter))

        except ImportError:
            # OTLP exporter not installed - telemetry disabled
            _initialized = True
            return False

        # Set as global provider
        trace.set_tracer_provider(provider)

        # Get tracer
        _tracer = trace.get_tracer("terminaluse.cli", service_version)

        _initialized = True
        return True

    except Exception:
        # Silently fail - telemetry should never break CLI
        _initialized = True
        return False


def get_tracer() -> "Tracer":
    """Get the CLI tracer instance.

    Returns a no-op tracer if telemetry is not initialized.
    """
    from opentelemetry import trace

    if not _initialized:
        init_cli_telemetry()

    return _tracer or trace.get_tracer("terminaluse.cli")


@contextmanager
def cli_command_span(
    command: str,
    print_trace_id: bool = False,
) -> Iterator["Span"]:
    """Context manager for CLI command spans.

    Creates a span for a CLI command with proper attributes and error handling.

    Args:
        command: The command name (e.g., "tasks.create", "deploy")
        print_trace_id: If True, prints the trace ID for debugging

    Yields:
        The active span for the command

    Example:
        with cli_command_span("tasks.create") as span:
            span.set_attribute("task.name", "my-task")
            # ... command logic
    """
    from opentelemetry import trace

    tracer = get_tracer()

    with tracer.start_as_current_span(
        f"cli.{command}",
        kind=trace.SpanKind.CLIENT,
        attributes={
            "cli.command": command,
            "cli.tool": "tu",
        },
    ) as span:
        if print_trace_id and span.is_recording():
            trace_id = format(span.get_span_context().trace_id, "032x")
            print(f"Trace ID: {trace_id}")

        try:
            yield span
        except Exception as e:
            span.set_status(trace.Status(trace.StatusCode.ERROR, str(e)))
            span.record_exception(e)
            raise


def inject_trace_context(headers: dict) -> dict:
    """Inject W3C trace context into HTTP headers.

    Adds traceparent and tracestate headers for distributed tracing.

    Args:
        headers: Existing headers dict (will be modified in place)

    Returns:
        The headers dict with trace context injected
    """
    try:
        from opentelemetry.propagate import inject

        inject(headers)
    except Exception:
        # Silently fail - don't break requests if propagation fails
        pass
    return headers


def get_trace_headers() -> dict:
    """Get trace context headers for the current span.

    Returns:
        Dict with traceparent/tracestate headers, or empty dict if no active span
    """
    headers: dict = {}
    inject_trace_context(headers)
    return headers



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.publish_raw_event_response import PublishRawEventResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRawEventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def publish_raw_event(
        self,
        *,
        raw_event: typing.Dict[str, typing.Any],
        sdk_type: str,
        sdk_version: str,
        task_id: str,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PublishRawEventResponse]:
        """
        Publish a raw SDK event for transformation and streaming.

            This endpoint replaces the deprecated `/stream-events` endpoint.
            Events are transformed via SDK-specific adapters (based on `sdk_type` and `sdk_version`)
            and the resulting stream parts are published to Redis for real-time consumption.

            Messages that should be stored (e.g., AssistantMessage, UserMessage) are
            automatically persisted to MongoDB for later retrieval.

            **Supported SDK Types:**
            - `claude`: Claude Agent SDK messages (StreamEvent, AssistantMessage, etc.)
            - `codex`: Codex SDK thread events (`item.*`, `turn.*`, `thread.*`)
            - `terminal_use`: Direct UIMessage format (Vercel-compatible)

            **What gets stored:**
            - AssistantMessage, UserMessage, ResultMessage, SystemMessage

            **What does NOT get stored:**
            - StreamEvent (ephemeral streaming deltas)

            Requires `update` permission on the task.

        Parameters
        ----------
        raw_event : typing.Dict[str, typing.Any]
            The raw SDK event payload

        sdk_type : str
            SDK type identifier (e.g., 'claude', 'codex', 'terminal_use')

        sdk_version : str
            SDK version in semver format (e.g., '0.1.22')

        task_id : str
            Task ID to publish the event to

        idempotency_key : typing.Optional[str]
            Optional idempotency key for request-level deduplication

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PublishRawEventResponse]
            Successful Response
        """
        _request_options_with_retries_disabled: typing.Optional[RequestOptions] = (
            {**request_options, "max_retries": 0} if request_options is not None else {"max_retries": 0}
        )
        _response = self._client_wrapper.httpx_client.request(
            "raw-events",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "raw_event": raw_event,
                "sdk_type": sdk_type,
                "sdk_version": sdk_version,
                "task_id": task_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=_request_options_with_retries_disabled,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PublishRawEventResponse,
                    parse_obj_as(
                        type_=PublishRawEventResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRawEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def publish_raw_event(
        self,
        *,
        raw_event: typing.Dict[str, typing.Any],
        sdk_type: str,
        sdk_version: str,
        task_id: str,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PublishRawEventResponse]:
        """
        Publish a raw SDK event for transformation and streaming.

            This endpoint replaces the deprecated `/stream-events` endpoint.
            Events are transformed via SDK-specific adapters (based on `sdk_type` and `sdk_version`)
            and the resulting stream parts are published to Redis for real-time consumption.

            Messages that should be stored (e.g., AssistantMessage, UserMessage) are
            automatically persisted to MongoDB for later retrieval.

            **Supported SDK Types:**
            - `claude`: Claude Agent SDK messages (StreamEvent, AssistantMessage, etc.)
            - `codex`: Codex SDK thread events (`item.*`, `turn.*`, `thread.*`)
            - `terminal_use`: Direct UIMessage format (Vercel-compatible)

            **What gets stored:**
            - AssistantMessage, UserMessage, ResultMessage, SystemMessage

            **What does NOT get stored:**
            - StreamEvent (ephemeral streaming deltas)

            Requires `update` permission on the task.

        Parameters
        ----------
        raw_event : typing.Dict[str, typing.Any]
            The raw SDK event payload

        sdk_type : str
            SDK type identifier (e.g., 'claude', 'codex', 'terminal_use')

        sdk_version : str
            SDK version in semver format (e.g., '0.1.22')

        task_id : str
            Task ID to publish the event to

        idempotency_key : typing.Optional[str]
            Optional idempotency key for request-level deduplication

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PublishRawEventResponse]
            Successful Response
        """
        _request_options_with_retries_disabled: typing.Optional[RequestOptions] = (
            {**request_options, "max_retries": 0} if request_options is not None else {"max_retries": 0}
        )
        _response = await self._client_wrapper.httpx_client.request(
            "raw-events",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "raw_event": raw_event,
                "sdk_type": sdk_type,
                "sdk_version": sdk_version,
                "task_id": task_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=_request_options_with_retries_disabled,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PublishRawEventResponse,
                    parse_obj_as(
                        type_=PublishRawEventResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.event import Event
from .raw_client import AsyncRawEventsClient, RawEventsClient


class EventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEventsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        task_id: str,
        agent_id: str,
        last_processed_event_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Event]:
        """
        List events for a specific task and agent.

        Optionally filter for events after a specific sequence ID.
        Results are ordered by sequence_id.

        Parameters
        ----------
        task_id : str
            The task ID

        agent_id : str
            The agent ID

        last_processed_event_id : typing.Optional[str]
            Optional event ID to get events after this ID

        limit : typing.Optional[int]
            Optional limit on number of results

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Event]
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.events.list(
            task_id="task_id",
            agent_id="agent_id",
        )
        """
        _response = self._raw_client.list(
            task_id=task_id,
            agent_id=agent_id,
            last_processed_event_id=last_processed_event_id,
            limit=limit,
            request_options=request_options,
        )
        return _response.data

    def retrieve(self, event_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Event:
        """
        Parameters
        ----------
        event_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Event
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.events.retrieve(
            event_id="event_id",
        )
        """
        _response = self._raw_client.retrieve(event_id, request_options=request_options)
        return _response.data


class AsyncEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEventsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        task_id: str,
        agent_id: str,
        last_processed_event_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Event]:
        """
        List events for a specific task and agent.

        Optionally filter for events after a specific sequence ID.
        Results are ordered by sequence_id.

        Parameters
        ----------
        task_id : str
            The task ID

        agent_id : str
            The agent ID

        last_processed_event_id : typing.Optional[str]
            Optional event ID to get events after this ID

        limit : typing.Optional[int]
            Optional limit on number of results

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Event]
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.events.list(
                task_id="task_id",
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            task_id=task_id,
            agent_id=agent_id,
            last_processed_event_id=last_processed_event_id,
            limit=limit,
            request_options=request_options,
        )
        return _response.data

    async def retrieve(self, event_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Event:
        """
        Parameters
        ----------
        event_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Event
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.events.retrieve(
                event_id="event_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(event_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SystemFolderResponse(UniversalBaseModel):
    """
    Response model for a task's system folder record.
    """

    archive_checksum: typing.Optional[str] = pydantic.Field(default=None)
    """
    SHA256 checksum of the archive. NULL means never synced.
    """

    archive_size_bytes: typing.Optional[int] = pydantic.Field(default=None)
    """
    Size of the archive in bytes after last sync.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when this record was created.
    """

    folder_type: str = pydantic.Field()
    """
    The type of system folder (e.g., 'dot_claude').
    """

    id: str = pydantic.Field()
    """
    The unique identifier of this record.
    """

    last_synced_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Timestamp of the last successful sync operation.
    """

    sync_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    Current sync status: PENDING, SYNCING, SUCCESS, or FAILED.
    """

    task_id: str = pydantic.Field()
    """
    The task this system folder belongs to.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when this record was last updated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.create_organization_invitation_response import CreateOrganizationInvitationResponse
from ..types.delete_response import DeleteResponse
from ..types.http_validation_error import HttpValidationError
from ..types.organization_invitation_response import OrganizationInvitationResponse
from ..types.organization_member_response import OrganizationMemberResponse
from ..types.organization_response import OrganizationResponse
from .types.create_organization_invitation_request_role_slug import CreateOrganizationInvitationRequestRoleSlug
from .types.update_organization_member_request_role_slug import UpdateOrganizationMemberRequestRoleSlug

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawOrganizationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[OrganizationResponse]]:
        """
        List all organizations the current user belongs to.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[OrganizationResponse]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/organizations",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[OrganizationResponse],
                    parse_obj_as(
                        type_=typing.List[OrganizationResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def retrieve_current(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[OrganizationResponse]:
        """
        Get the current user's organization info.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[OrganizationResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/organizations/current",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrganizationResponse,
                    parse_obj_as(
                        type_=OrganizationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_invitations(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[OrganizationInvitationResponse]]:
        """
        List pending invitations for the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[OrganizationInvitationResponse]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/organizations/invitations",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[OrganizationInvitationResponse],
                    parse_obj_as(
                        type_=typing.List[OrganizationInvitationResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_invitation(
        self,
        *,
        email: str,
        role_slug: typing.Optional[CreateOrganizationInvitationRequestRoleSlug] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateOrganizationInvitationResponse]:
        """
        Invite a user to the current organization. Idempotent for existing pending invites by email.

        Parameters
        ----------
        email : str
            Invitee email

        role_slug : typing.Optional[CreateOrganizationInvitationRequestRoleSlug]
            Organization role slug for the invited member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateOrganizationInvitationResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/organizations/invitations",
            method="POST",
            json={
                "email": email,
                "role_slug": role_slug,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateOrganizationInvitationResponse,
                    parse_obj_as(
                        type_=CreateOrganizationInvitationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def revoke_invitation(
        self, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[OrganizationInvitationResponse]:
        """
        Revoke a pending invitation in the current organization.

        Parameters
        ----------
        invitation_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[OrganizationInvitationResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/invitations/{jsonable_encoder(invitation_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrganizationInvitationResponse,
                    parse_obj_as(
                        type_=OrganizationInvitationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def resend_invitation(
        self, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[OrganizationInvitationResponse]:
        """
        Resend a pending invitation in the current organization.

        Parameters
        ----------
        invitation_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[OrganizationInvitationResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/invitations/{jsonable_encoder(invitation_id)}/resend",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrganizationInvitationResponse,
                    parse_obj_as(
                        type_=OrganizationInvitationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_members(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[OrganizationMemberResponse]]:
        """
        List active members in the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[OrganizationMemberResponse]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/organizations/members",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[OrganizationMemberResponse],
                    parse_obj_as(
                        type_=typing.List[OrganizationMemberResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_member(
        self,
        membership_id: str,
        *,
        role_slug: UpdateOrganizationMemberRequestRoleSlug,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[OrganizationMemberResponse]:
        """
        Update a member role in the current organization.

        Parameters
        ----------
        membership_id : str

        role_slug : UpdateOrganizationMemberRequestRoleSlug
            Organization role slug for the target member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[OrganizationMemberResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/members/{jsonable_encoder(membership_id)}",
            method="PUT",
            json={
                "role_slug": role_slug,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrganizationMemberResponse,
                    parse_obj_as(
                        type_=OrganizationMemberResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove_member(
        self, membership_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteResponse]:
        """
        Remove an active member from the current organization.

        Parameters
        ----------
        membership_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/members/{jsonable_encoder(membership_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteResponse,
                    parse_obj_as(
                        type_=DeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawOrganizationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[OrganizationResponse]]:
        """
        List all organizations the current user belongs to.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[OrganizationResponse]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/organizations",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[OrganizationResponse],
                    parse_obj_as(
                        type_=typing.List[OrganizationResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def retrieve_current(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[OrganizationResponse]:
        """
        Get the current user's organization info.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[OrganizationResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/organizations/current",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrganizationResponse,
                    parse_obj_as(
                        type_=OrganizationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_invitations(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[OrganizationInvitationResponse]]:
        """
        List pending invitations for the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[OrganizationInvitationResponse]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/organizations/invitations",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[OrganizationInvitationResponse],
                    parse_obj_as(
                        type_=typing.List[OrganizationInvitationResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_invitation(
        self,
        *,
        email: str,
        role_slug: typing.Optional[CreateOrganizationInvitationRequestRoleSlug] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateOrganizationInvitationResponse]:
        """
        Invite a user to the current organization. Idempotent for existing pending invites by email.

        Parameters
        ----------
        email : str
            Invitee email

        role_slug : typing.Optional[CreateOrganizationInvitationRequestRoleSlug]
            Organization role slug for the invited member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateOrganizationInvitationResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/organizations/invitations",
            method="POST",
            json={
                "email": email,
                "role_slug": role_slug,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateOrganizationInvitationResponse,
                    parse_obj_as(
                        type_=CreateOrganizationInvitationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def revoke_invitation(
        self, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[OrganizationInvitationResponse]:
        """
        Revoke a pending invitation in the current organization.

        Parameters
        ----------
        invitation_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[OrganizationInvitationResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/invitations/{jsonable_encoder(invitation_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrganizationInvitationResponse,
                    parse_obj_as(
                        type_=OrganizationInvitationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def resend_invitation(
        self, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[OrganizationInvitationResponse]:
        """
        Resend a pending invitation in the current organization.

        Parameters
        ----------
        invitation_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[OrganizationInvitationResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/invitations/{jsonable_encoder(invitation_id)}/resend",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrganizationInvitationResponse,
                    parse_obj_as(
                        type_=OrganizationInvitationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_members(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[OrganizationMemberResponse]]:
        """
        List active members in the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[OrganizationMemberResponse]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/organizations/members",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[OrganizationMemberResponse],
                    parse_obj_as(
                        type_=typing.List[OrganizationMemberResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_member(
        self,
        membership_id: str,
        *,
        role_slug: UpdateOrganizationMemberRequestRoleSlug,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[OrganizationMemberResponse]:
        """
        Update a member role in the current organization.

        Parameters
        ----------
        membership_id : str

        role_slug : UpdateOrganizationMemberRequestRoleSlug
            Organization role slug for the target member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[OrganizationMemberResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/members/{jsonable_encoder(membership_id)}",
            method="PUT",
            json={
                "role_slug": role_slug,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrganizationMemberResponse,
                    parse_obj_as(
                        type_=OrganizationMemberResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove_member(
        self, membership_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteResponse]:
        """
        Remove an active member from the current organization.

        Parameters
        ----------
        membership_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/members/{jsonable_encoder(membership_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteResponse,
                    parse_obj_as(
                        type_=DeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



"""Tests for output filter system."""

import re

from terminaluse.lib.utils.output import (
    BlockRemoveFilter,
    FilterPipeline,
    LineFilter,
    LinePrefixStripFilter,
    RegexRemoveFilter,
)


class TestRegexRemoveFilter:
    """Tests for RegexRemoveFilter."""

    def test_removes_matching_content(self) -> None:
        """Removes content matching the pattern."""
        pattern = re.compile(r"<tag>.*?</tag>", re.DOTALL)
        f = RegexRemoveFilter(pattern)

        result = f.apply("before <tag>secret</tag> after")
        assert result == "before  after"

    def test_removes_multiline_content(self) -> None:
        """Removes content spanning multiple lines."""
        pattern = re.compile(r"<block>.*?</block>", re.DOTALL)
        f = RegexRemoveFilter(pattern)

        text = "before\n<block>\nline1\nline2\n</block>\nafter"
        result = f.apply(text)
        assert result == "before\n\nafter"

    def test_no_match_unchanged(self) -> None:
        """Text without matches is unchanged."""
        pattern = re.compile(r"<tag>.*?</tag>")
        f = RegexRemoveFilter(pattern)

        result = f.apply("no tags here")
        assert result == "no tags here"


class TestLineFilter:
    """Tests for LineFilter."""

    def test_keeps_matching_lines(self) -> None:
        """Keeps lines where predicate returns True."""
        f = LineFilter(lambda line: "keep" in line)

        text = "keep this\nremove this\nkeep this too"
        result = f.apply(text)
        assert result == "keep this\nkeep this too"

    def test_removes_metadata_lines(self) -> None:
        """Removes lines starting with metadata prefixes."""

        def not_metadata(line: str) -> bool:
            return not line.strip().startswith("agentId:")

        f = LineFilter(not_metadata)

        text = "Hello\nagentId: abc123\nWorld"
        result = f.apply(text)
        assert result == "Hello\nWorld"

    def test_empty_result_if_all_filtered(self) -> None:
        """Returns empty string if all lines filtered."""
        f = LineFilter(lambda _: False)

        result = f.apply("line1\nline2\nline3")
        assert result == ""


class TestLinePrefixStripFilter:
    """Tests for LinePrefixStripFilter."""

    def test_strips_line_number_prefix(self) -> None:
        """Strips line number prefixes from each line."""
        pattern = re.compile(r"^\s*\d+→")
        f = LinePrefixStripFilter(pattern)

        text = "  42→content here\n  43→more content"
        result = f.apply(text)
        assert result == "content here\nmore content"

    def test_preserves_lines_without_prefix(self) -> None:
        """Lines without the prefix are preserved."""
        pattern = re.compile(r"^\s*\d+→")
        f = LinePrefixStripFilter(pattern)

        text = "no prefix here\n  42→has prefix"
        result = f.apply(text)
        assert result == "no prefix here\nhas prefix"


class TestBlockRemoveFilter:
    """Tests for BlockRemoveFilter."""

    def test_removes_simple_block(self) -> None:
        """Removes content between markers."""
        f = BlockRemoveFilter("<secret>", "</secret>")

        text = "before <secret>hidden</secret> after"
        result = f.apply(text)
        assert "hidden" not in result
        assert "before" in result
        assert "after" in result

    def test_removes_multiline_block(self) -> None:
        """Removes blocks spanning multiple lines."""
        f = BlockRemoveFilter("<remove>", "</remove>")

        text = "keep this\n<remove>\nhidden1\nhidden2\n</remove>\nkeep this too"
        result = f.apply(text)
        assert "hidden" not in result
        assert "keep this" in result
        assert "keep this too" in result

    def test_handles_same_line_markers(self) -> None:
        """Handles both markers on the same line."""
        f = BlockRemoveFilter("<x>", "</x>")

        text = "before <x>middle</x> after"
        result = f.apply(text)
        assert result == "before  after"

    def test_no_markers_unchanged(self) -> None:
        """Text without markers is unchanged."""
        f = BlockRemoveFilter("<x>", "</x>")

        text = "no markers here"
        result = f.apply(text)
        assert result == "no markers here"


class TestFilterPipeline:
    """Tests for FilterPipeline."""

    def test_applies_filters_in_order(self) -> None:
        """Filters are applied in sequence."""
        pipeline = FilterPipeline(
            [
                RegexRemoveFilter(re.compile(r"<a>.*?</a>")),
                RegexRemoveFilter(re.compile(r"<b>.*?</b>")),
            ]
        )

        text = "keep <a>remove</a> also <b>remove</b> keep"
        result = pipeline.apply(text)
        assert result == "keep  also  keep"

    def test_empty_pipeline(self) -> None:
        """Empty pipeline returns input unchanged."""
        pipeline = FilterPipeline()

        result = pipeline.apply("unchanged")
        assert result == "unchanged"

    def test_with_filter_creates_new_pipeline(self) -> None:
        """with_filter returns new pipeline, original unchanged."""
        original = FilterPipeline([])
        extended = original.with_filter(RegexRemoveFilter(re.compile(r"x")))

        assert len(original) == 0
        assert len(extended) == 1

    def test_with_filters_appends_multiple(self) -> None:
        """with_filters appends multiple filters."""
        pipeline = FilterPipeline([])
        extended = pipeline.with_filters(
            [
                RegexRemoveFilter(re.compile(r"a")),
                RegexRemoveFilter(re.compile(r"b")),
            ]
        )

        assert len(extended) == 2

    def test_complex_pipeline(self) -> None:
        """Complex pipeline with multiple filter types."""
        pipeline = FilterPipeline(
            [
                RegexRemoveFilter(re.compile(r"<system>.*?</system>", re.DOTALL)),
                LineFilter(lambda line: "metadata:" not in line),
                LinePrefixStripFilter(re.compile(r"^\d+\|")),
            ]
        )

        text = "1|content\n<system>hidden</system>\nmetadata: value\n2|more"
        result = pipeline.apply(text)
        assert result == "content\n\nmore"



"""Telemetry configuration for the TerminalUse SDK.

Telemetry (traces, metrics, log correlation) is automatically configured
when you create an AgentServer. No manual setup required.

Control via environment variables:
- TERMINALUSE_TELEMETRY=false: Disable telemetry entirely
- TERMINALUSE_AUTO_INSTRUMENT=false: Disable auto-instrumentation (manual spans only)

Prerequisites (set by platform when deploying agents):
- TERMINALUSE_BASE_URL: Nucleus API URL
- TERMINALUSE_AGENT_API_KEY: Agent authentication key

What's automatic with auto-instrumentation (default):
- HTTP server spans for all incoming FastAPI requests
- HTTP client spans for all outgoing httpx requests
- W3C trace context propagation
- Log correlation (logs include trace_id/span_id)

Manual spans (works with or without auto-instrumentation):
    from opentelemetry import trace

    tracer = trace.get_tracer(__name__)
    with tracer.start_as_current_span("my_operation"):
        do_work()

Advanced: If you need to configure telemetry before AgentServer
(rare - usually not needed):
    from terminaluse.lib.telemetry import configure_agent_telemetry
    configure_agent_telemetry(auto_instrument=True)
"""

from terminaluse.lib.telemetry.otel_config import configure_agent_telemetry

__all__ = ["configure_agent_telemetry"]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.metrics_ingestion_response import MetricsIngestionResponse
from ..types.trace_ingestion_response import TraceIngestionResponse
from .raw_client import AsyncRawTelemetryClient, RawTelemetryClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TelemetryClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTelemetryClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTelemetryClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTelemetryClient
        """
        return self._raw_client

    def agent_metrics_ingest(
        self,
        *,
        resource_metrics: typing.Sequence[typing.Dict[str, typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MetricsIngestionResponse:
        """
        Ingest OpenTelemetry metrics from an agent. Requires agent API key authentication.

        Parameters
        ----------
        resource_metrics : typing.Sequence[typing.Dict[str, typing.Any]]
            Array of ResourceMetrics as per OTLP JSON format

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MetricsIngestionResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.telemetry.agent_metrics_ingest(
            resource_metrics=[{"key": "value"}],
        )
        """
        _response = self._raw_client.agent_metrics_ingest(
            resource_metrics=resource_metrics, request_options=request_options
        )
        return _response.data

    def traces_ingest(
        self,
        *,
        resource_spans: typing.Sequence[typing.Dict[str, typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraceIngestionResponse:
        """
        Ingest OpenTelemetry traces from an agent. Requires agent API key authentication.

        Parameters
        ----------
        resource_spans : typing.Sequence[typing.Dict[str, typing.Any]]
            Array of ResourceSpans as per OTLP JSON format

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraceIngestionResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.telemetry.traces_ingest(
            resource_spans=[{"key": "value"}],
        )
        """
        _response = self._raw_client.traces_ingest(resource_spans=resource_spans, request_options=request_options)
        return _response.data


class AsyncTelemetryClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTelemetryClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTelemetryClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTelemetryClient
        """
        return self._raw_client

    async def agent_metrics_ingest(
        self,
        *,
        resource_metrics: typing.Sequence[typing.Dict[str, typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MetricsIngestionResponse:
        """
        Ingest OpenTelemetry metrics from an agent. Requires agent API key authentication.

        Parameters
        ----------
        resource_metrics : typing.Sequence[typing.Dict[str, typing.Any]]
            Array of ResourceMetrics as per OTLP JSON format

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MetricsIngestionResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.telemetry.agent_metrics_ingest(
                resource_metrics=[{"key": "value"}],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.agent_metrics_ingest(
            resource_metrics=resource_metrics, request_options=request_options
        )
        return _response.data

    async def traces_ingest(
        self,
        *,
        resource_spans: typing.Sequence[typing.Dict[str, typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TraceIngestionResponse:
        """
        Ingest OpenTelemetry traces from an agent. Requires agent API key authentication.

        Parameters
        ----------
        resource_spans : typing.Sequence[typing.Dict[str, typing.Any]]
            Array of ResourceSpans as per OTLP JSON format

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TraceIngestionResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.telemetry.traces_ingest(
                resource_spans=[{"key": "value"}],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.traces_ingest(resource_spans=resource_spans, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_organization_invitation_response import CreateOrganizationInvitationResponse
from ..types.delete_response import DeleteResponse
from ..types.organization_invitation_response import OrganizationInvitationResponse
from ..types.organization_member_response import OrganizationMemberResponse
from ..types.organization_response import OrganizationResponse
from .raw_client import AsyncRawOrganizationsClient, RawOrganizationsClient
from .types.create_organization_invitation_request_role_slug import CreateOrganizationInvitationRequestRoleSlug
from .types.update_organization_member_request_role_slug import UpdateOrganizationMemberRequestRoleSlug

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OrganizationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawOrganizationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawOrganizationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawOrganizationsClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[OrganizationResponse]:
        """
        List all organizations the current user belongs to.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[OrganizationResponse]
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.organizations.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def retrieve_current(self, *, request_options: typing.Optional[RequestOptions] = None) -> OrganizationResponse:
        """
        Get the current user's organization info.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.organizations.retrieve_current()
        """
        _response = self._raw_client.retrieve_current(request_options=request_options)
        return _response.data

    def list_invitations(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[OrganizationInvitationResponse]:
        """
        List pending invitations for the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[OrganizationInvitationResponse]
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.organizations.list_invitations()
        """
        _response = self._raw_client.list_invitations(request_options=request_options)
        return _response.data

    def create_invitation(
        self,
        *,
        email: str,
        role_slug: typing.Optional[CreateOrganizationInvitationRequestRoleSlug] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationInvitationResponse:
        """
        Invite a user to the current organization. Idempotent for existing pending invites by email.

        Parameters
        ----------
        email : str
            Invitee email

        role_slug : typing.Optional[CreateOrganizationInvitationRequestRoleSlug]
            Organization role slug for the invited member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationInvitationResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.organizations.create_invitation(
            email="email",
        )
        """
        _response = self._raw_client.create_invitation(
            email=email, role_slug=role_slug, request_options=request_options
        )
        return _response.data

    def revoke_invitation(
        self, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationInvitationResponse:
        """
        Revoke a pending invitation in the current organization.

        Parameters
        ----------
        invitation_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationInvitationResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.organizations.revoke_invitation(
            invitation_id="invitation_id",
        )
        """
        _response = self._raw_client.revoke_invitation(invitation_id, request_options=request_options)
        return _response.data

    def resend_invitation(
        self, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationInvitationResponse:
        """
        Resend a pending invitation in the current organization.

        Parameters
        ----------
        invitation_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationInvitationResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.organizations.resend_invitation(
            invitation_id="invitation_id",
        )
        """
        _response = self._raw_client.resend_invitation(invitation_id, request_options=request_options)
        return _response.data

    def list_members(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[OrganizationMemberResponse]:
        """
        List active members in the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[OrganizationMemberResponse]
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.organizations.list_members()
        """
        _response = self._raw_client.list_members(request_options=request_options)
        return _response.data

    def update_member(
        self,
        membership_id: str,
        *,
        role_slug: UpdateOrganizationMemberRequestRoleSlug,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationMemberResponse:
        """
        Update a member role in the current organization.

        Parameters
        ----------
        membership_id : str

        role_slug : UpdateOrganizationMemberRequestRoleSlug
            Organization role slug for the target member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationMemberResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.organizations.update_member(
            membership_id="membership_id",
            role_slug="admin",
        )
        """
        _response = self._raw_client.update_member(membership_id, role_slug=role_slug, request_options=request_options)
        return _response.data

    def remove_member(
        self, membership_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteResponse:
        """
        Remove an active member from the current organization.

        Parameters
        ----------
        membership_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteResponse
            Successful Response

        Examples
        --------
        from terminaluse import TerminalUseBase

        client = TerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.organizations.remove_member(
            membership_id="membership_id",
        )
        """
        _response = self._raw_client.remove_member(membership_id, request_options=request_options)
        return _response.data


class AsyncOrganizationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawOrganizationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawOrganizationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawOrganizationsClient
        """
        return self._raw_client

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[OrganizationResponse]:
        """
        List all organizations the current user belongs to.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[OrganizationResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.organizations.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def retrieve_current(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationResponse:
        """
        Get the current user's organization info.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.organizations.retrieve_current()


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve_current(request_options=request_options)
        return _response.data

    async def list_invitations(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[OrganizationInvitationResponse]:
        """
        List pending invitations for the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[OrganizationInvitationResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.organizations.list_invitations()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_invitations(request_options=request_options)
        return _response.data

    async def create_invitation(
        self,
        *,
        email: str,
        role_slug: typing.Optional[CreateOrganizationInvitationRequestRoleSlug] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationInvitationResponse:
        """
        Invite a user to the current organization. Idempotent for existing pending invites by email.

        Parameters
        ----------
        email : str
            Invitee email

        role_slug : typing.Optional[CreateOrganizationInvitationRequestRoleSlug]
            Organization role slug for the invited member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationInvitationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.organizations.create_invitation(
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_invitation(
            email=email, role_slug=role_slug, request_options=request_options
        )
        return _response.data

    async def revoke_invitation(
        self, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationInvitationResponse:
        """
        Revoke a pending invitation in the current organization.

        Parameters
        ----------
        invitation_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationInvitationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.organizations.revoke_invitation(
                invitation_id="invitation_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.revoke_invitation(invitation_id, request_options=request_options)
        return _response.data

    async def resend_invitation(
        self, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationInvitationResponse:
        """
        Resend a pending invitation in the current organization.

        Parameters
        ----------
        invitation_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationInvitationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.organizations.resend_invitation(
                invitation_id="invitation_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.resend_invitation(invitation_id, request_options=request_options)
        return _response.data

    async def list_members(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[OrganizationMemberResponse]:
        """
        List active members in the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[OrganizationMemberResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.organizations.list_members()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_members(request_options=request_options)
        return _response.data

    async def update_member(
        self,
        membership_id: str,
        *,
        role_slug: UpdateOrganizationMemberRequestRoleSlug,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationMemberResponse:
        """
        Update a member role in the current organization.

        Parameters
        ----------
        membership_id : str

        role_slug : UpdateOrganizationMemberRequestRoleSlug
            Organization role slug for the target member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationMemberResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.organizations.update_member(
                membership_id="membership_id",
                role_slug="admin",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_member(
            membership_id, role_slug=role_slug, request_options=request_options
        )
        return _response.data

    async def remove_member(
        self, membership_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteResponse:
        """
        Remove an active member from the current organization.

        Parameters
        ----------
        membership_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteResponse
            Successful Response

        Examples
        --------
        import asyncio

        from terminaluse import AsyncTerminalUseBase

        client = AsyncTerminalUseBase(
            agent_api_key="YOUR_AGENT_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.organizations.remove_member(
                membership_id="membership_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove_member(membership_id, request_options=request_options)
        return _response.data
